function Result=Experiment1a(varargin)
% Simulated experiment.
%
% Simulated robot moving on a couple of concentric ellipses.
%
% The sensor parameters are set to define the plot in Fig. 1(a) in the 
% paper. 
% Note that this implements the standard Pose SLAM algorithm, without using
% the loop closure state recovery strategy introduced in the paper.
% Thus, whenever a loop is closed we invert the information matrix to
% recover the covariance to keep the blocks necessaries for data necessary 
% for data association and to recover the new mean.
% Note, though, that recovering the whole covariance matrix only works for
% small problems due to memory requirements.
%
% The (optional) paremeters of this function are
%    stateRecoveryMethod:   
%            0 -> Get Sigma column by column (default)
%            1 -> Get whole Sigma 
%            2 -> Our method 
%    nSteps: How many steps to simulate? The default value is 0, i.e.,
%            simulate as far as sensor readings are available.
%            Partial runs can be continued with ContinueSimulation.
%
% By directly editing this file you can execute the simulation with
% different filters and loop detection procedures.
%
% The possible filters are
%      - EKF: Extended Kalman Filter. It uses the first available sensor
%        reading to extend the state (Prediction) and the rest to
%        improve its estimation (Update). The Update is also used when
%        a loop is closed. This filter only works fine on small problems.
%
%      - TRO: Our filter. It can be seen as an EIF where we take
%        advantage that only very few loops are closed. Thanks to this,
%        most of the time the filter is executed in constant time.
%        Moreover, the TRO filter implements a strategy to remove
%        redundant poses: poses that, with high probability
%        are close to the current pose and that do not offer qualitatively 
%        more inmore information than what is already available in the 
%        filter.
%
%  As far as loop detection procedures, you can use
%      - NoLoops: The filter proposes no loop closures. Consequently, the
%        simulation proceeds in open loop.
%
%      - AllLoops: The filter proposes to close all possible loops (i.e.,
%        loops linking the current robot pose with all the previous
%        ones). In this case the loop detection completely relies on the
%        sensors.
%
%      - DetectLoops: The filter detects the poses that are with high 
%        probability closer than a given threshold to the current robot 
%        pose. 
%
%      - TreeDetectLoops: Uses a tree structure that offers searches in 
%        logarithmic time instead of the plain linear algorithm used by 
%        DetectLoops. Note that the tree is only clearly advantageous in
%        large problems.
%
% Returns:
%     Result: A structure with all the parameters used in the experiment,
%             the resulting filter structure, and the execution times.
%
% See also ContinueSimulation.
 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Process the input parameters
  
  % For TRO_Filter: state update method when in loop closure
  %   0 -> Get Sigma column by column
  %   1 -> Get whole Sigma
  %   2 -> Our method
  if nargin>0
    stateRecoveryMethod=varargin{1};
  else
    stateRecoveryMethod=0;
  end

  % Steps to simulate (use 0 to execute all steps)
  if nargin>1
    Parameters.maxSteps=varargin{2};
  else
    Parameters.maxSteps=0;
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Now set the filter type and the data association procedure to use.
  
  % Type of filter to use: EKF, TRO.
  %   Note that EKF works fine for this small example but not for larger 
  %   ones and that the stateRecovery parameter only applies to the TRO 
  %   filter.
  
  % filterType='EKF';
  filterType='TRO';
  
  % Loop detection procedure to use:
  %  * NoLoops (simulation in open loop)
  %  * AllLoops (sensor-based loop clousure)
  %  * DetectLoops (lineal search), 
  %  * treeDetectLoops (interval-based tree search introduced in the paper)
  %                    (This is only worth for large problems)
  
  % Parameters.loopClosure='NoLoops';
  % Parameters.loopClosure='AllLoops';
  Parameters.loopClosure='DetectLoops';
  % Parameters.loopClosure='TreeDetectLoops';
  
  % Distance prob. thresholds (only used for DetetLoops and
  % TreeDetectLoops).
  % - If the probability of two poses of being closer than  'matchArea'
  %   is larger than pdRange(1) we try to create a loop (if the information
  %   gain criterion also confirms it, see below)
  % - If the probability of two poses of being closer that  'matchArea'
  %   is larger than pdRange(2) the poses are close enough for one of them
  %   to be redundant  (if the information gain criterion also confirms it, 
  %   see below)
  % This is 's' in the paper.
  Parameters.pdRange=[0.1 0.1];

  % information gain range (only used for DetetLoops and TreeDetectLoops).
  % - if the information gain is larger than igRange(2) we try to create a
  %   loop
  % - if the information gain is smaller than igRange(1) the pose is
  %   redundant and it can be overwrite next time slice
  % This is 'g' in the paper.
  Parameters.igRange=[0 0];
  
  % Set up  the trajectory
  Te1=Trajectory(pi/2,pi/2-2*pi,10,6,70); % First ellipse
  Te2=Trajectory(pi/2,pi/2-2*pi,20,6,100); % Second ellipse
  T=Trajectory(Te1,Te2); 

  % Error in odometry 
  % 5% of the displacement, 1 degree in rotation
  % Values taken from "Data Association in O(n) for Divide and
  % Conquer Slam" by L.M Paz, J. Guivant, J.D. Tardos and J. Neira.
  ad=0.05*get(T,'averageDisplacement');
  SnOdometry=[[ad(1)^2,0,0];[0,ad(2)^2,0];[0,0,(1*pi/180)^2]]; 

  % Error in simulated "vision" (odometry + loop closure)
  SnVision=[[0.2^2,0,0];[0,0.2^2,0];[0,0,(0.5*pi/180)^2]];
  
  % Covariance of the 1st pose. The mean of the first pose in taken from
  % the trajectory (i.e., the ground truth)
  initialUncertainty=[[0.1^2,0,0];[0,0.1^2,0];[0,0,(5*pi/180)^2]];

  % Area around one pose where new images
  % tipycally match the current image
  % These are the  v_r in the paper.
  Parameters.matchArea=[3; 3; (15*pi/180)];
  
  % Noise used to simulate loop closures, i.e., to compute the
  % expected information gain of a candidate loop before actually
  % registering the sensor readings
  Parameters.noiseLoopClosure=SnVision;

  % Not to print info while solving
  Parameters.quiet=false;
  
  % True if you want graphical output while simulating
  Parameters.plot=true;

  % Use ground truth (this is used to evalute the NEES)
  Parameters.groundTruth=T;
  
  % Area where the robot will move. This is only used for plotting
  Parameters.Domain=get(T,'boundingBox');
  
  % Create the sensors: odometry and vision like sensors
  Sensors{1}=SimOdometry(SnOdometry,T);
  Sensors{2}=SimOdometryWithLoopClosure(SnVision,T,Parameters.matchArea);

  % Assemble a robot with the just defined sensors
  R=Robot('SimulatedRobot',Sensors);

  % Define the inital robot position: We use the initial position of the
  % true trajectory as an initial robot position. This is the ground truth
  % since this experiment is all simulation.
  initialEstimation=Gaussian(double(T{1}),initialUncertainty);

  % Initialize the selected filter
  switch filterType
    case 'EKF'
      F=P_EKF(initialEstimation);
    case 'TRO'
      F=P_TRO(initialEstimation,stateRecoveryMethod);
    otherwise
      error('Unknown filter type');
  end
  
  % and launch the simulation
  Result=Simulation(F,R,Parameters);
  

