function EKF=Update(EKF,step,SR,Sn,overwritePose)
% Adds a new pose to an EKF filter.
% 
% Updates an 'Extended Kalman Filter' adding a new pose to the estimated
% trajectory. It updates the mean and the covariance matrix.
%
% Parameters:
%     EKF: The filter.
%     step: Time slice.
%     SR: The sensor readings.
%     Sn: The noise of the sensors.
%     overwritePose: Not used in the 'Extended Kalman Filter'.
% Returns:
%     EKF: The updated 'Extended Kalman Filter'.
%
% The first sensor reading is used to extend the state (i.e., the
% trajectory) using EKF_Prediction, and the remaining ones are used to
% improve the estimation of the current robot's pose using EKF_Update.
%
% See also EKF_Prediction, EKF_Update

  % overwritePose is not implemented in EkF. We keep the parameter
  % for compatibility with P_TRO filter .
  
  ns=size(SR,2); % number of sensor readings
  
  % First valid sensor reading is used for prediction (i.e., to extend the
  % state), and the rest of valid sensor readings for update.
  first=true;
  for i=1:ns
    d=SR{i};
    if ~NullPose(d)
      if first
        EKF=EKF_Prediction(EKF,d,Sn{i});
        
        EKF.P_Filter=Update(EKF.P_Filter,step,false);
        
        nStates=get(EKF.P_Filter,'nStates');
        first=false;
      else
        EKF=EKF_Update(EKF,nStates-1,d,Sn{i});
      end
    end
  end
  if first
    error('EKF step without any valid sensor reading');
  end
