function EKF=EKF_Update(EKF,i,d,R)
% EKF Update step.
%
% This is used when establishing a loop between the current robot pose
% and one of the previous poses, i. This can be the previous pose or one of
% the poses before it. In the first case we are basically forming a local
% loop with limited effects (only affects the current robot pose) and
% in the second we have a loop that can affect a significant part of the
% trajectory.
%
% Parameters
%      EKF: The filter to update.
%      i; The pose to be linked with the current robot's pose to form a
%         loop.
%      d: Displacement from pose 'i' to the current robot's pose 'n'
%         according to the sensors readings.
%      R: Noise of the sensor
% Outputs
%      EKF: The updated filter.
%
% See also EKF_Prediction, TRO_Prediction, TRO_Update.

  % the dispalcement 'd' MUST be from 'i' to 'n'
  nStates=get(EKF.P_Filter,'nStates');
  
  if i==nStates
    error('Self loop in EKF Update')
  end
  
  m=get(EKF.P_Filter,'dim'); % Dimensionality of the pose space
  rangeI=m*(i-1)+1:m*i;
  rangeN=m*(nStates-1)+1:m*nStates;
  
  ne=size(EKF.mu,1); % number of elements in the state (m per pose)
  
  mI=EKF.mu(rangeI);
  Pi=Pose(mI);
  
  mN=EKF.mu(rangeN);
  Pn=Pose(mN);
  
  y=double(d-Absolute2Relative(Pi,Pn));
  
  H=sparse(zeros(m,ne));
  
  [Ji Jn]=Absolute2RelativeJacobian(Pi,Pn);
  H(:,rangeI)=Ji;
  H(:,rangeN)=Jn;
  
  SHt=EKF.Sigma*H';
  
  S=H*SHt+R;
  S=(S+S')/2; % This enforces symmetry and enhances numerical stability
  
  L=chol(S);
  iL=inv(L);
  W1=SHt*iL;
  
  EKF.mu=EKF.mu + W1*iL'*y;
  
  EKF.Sigma=EKF.Sigma - W1*W1';
