function Result=Simulation(F,R,Parameters)
% Simulates a robot performing Pose SLAM.
%
% This is basically the MatLab implementation of algoritm 4 in the paper.
% This function is used to simulate from scratch and to continue a 
% previously started simulation.
%
%   Parameters:
%     F: the P_Filter to use to estimate the trajectory of the robot.
%     R: The robot to use in the simulation
%     Parameters.maxSteps: Number of steps to run the simulation. 
%             Set to 0 to use  as many steps as sensor readings available.
%             This parameter and the use of this function as well as
%             ContinueSimulation allow to run a full simulation in
%             separate phases.
%     Parameters.plot: true if the trajectory as estimated by the filter 
%                  has to be plotted while simulating.
%     Parameters.plotSensors: Array of booleans indicating which sensors 
%                  are to be plotted along with the filter information. 
%                  Right now only RealVison and RealLaser sensors have 
%                  non-void Plot functions. In the first case the images 
%                  of the cameras are shown at the right of the 
%                  trajectory and in the second case the laser readings 
%                  are plotted at the corresponding pose.
%     Parameters.Domain: Area where the robot is expected to move. Used 
%                  to define the tree of poses and the axis of the plot.
%     Parameters.loopClosure: Selects the method to determine the loop 
%                  closure points.
%     The rest of parametres depend on the 'loopClosure' value
%       For 'NoLoops' or 'AllLoos' no more parameters are needed.
%       For 'DetectLoops'or 'TreeDetectLoops'  we have to provide
%          - Parameters.matchArea: Area arround a pose where it is likely 
%            to get similar sensor readings (sensor readings that can be  
%            registered to the sensor readings of the current pose).
%            This is 'v_r' in the paper.
%            Todo: integrate this threshold in the sensor information.
%          - Parameters.loopNoise: Typical noise of the sensors closing 
%            loops. The sensors that will be used to register poses.
%            This is \bar{\Sigma_y} in the paper.
%          - Parameters.pdRange: An interval [p1,p2]. If a pose has a 
%            probability higher   p1 of being in the match area of the 
%            current robot's pose it is considered for a loop (if the 
%            information test below agrees to do so). If the pose has 
%            probability higher than p2 of being in the match area of the 
%            current robot's pose the current pose is considered for 
%            removal (see information test below)
%            This is 's' in the paper. However here we allow to define two
%            values for it.
%          - Parameters.igRange: An interval [ig1,ig2]. If the information 
%            gain of apossible loop closure (suggested by the distance 
%            criterion) is above ig2 the loops closure is considered 
%            informativeenough. If the information gain is below ig1, the 
%            current robot pose is considered redundant (it can be removed 
%            since there is a very similar pose already in the trajectory).
%            The information criterion is only considered after the
%            distance one.
%            This is 'g' in the paper. However we allow to define two
%            values for it.
%
%   Outputs:
%     Results: A structure with the Filter, the Robot, the Parameters and
%              some outputs (time, inf. gain,...)

  if ~Parameters.quiet
    fprintf('Simulating: \n');
  end
  
  % Robot's number of sensors
  Sensors=get(R,'sensors');
  numSensors=size(Sensors,2);
 
  pt=InitFilterPlot(F,Parameters);
  
  % Default output is a tree from the filter data (non-empty if we
  % are continuing a previously stopped simulation).
  if strcmp(Parameters.loopClosure,'TreeDetectLoops')
    pd=GetPoseData(F);
    T=BTree({pd{1:(end-1)}});
  else
    T=0;
   end
  
  % sometimes, when detecting possible points for loop closure we
  % detect that current pose is redundant (i.e., we already have
  % a nearby pose in the filter). If so, next pose to be added
  % replaces the current one. This helps to control the size
  % of the filter.
  redundantPose=false;
  
  % To gather estatistics about the loop closure
  NPL=0; % number of proposed loops
  NCL=0; % number of actually closed loops
  
  % To gather statistics about execution time
  timeSA=0; % Time used in state augmentation (Update)
  timeLC=0; % Time used in loop closure (UpdateLoop)
  timeS=0;  % Time used searching for neighbours
  timeT=0;  % Time used inserting in the tree and re-building it 
  
  totalIG=0; % total information gain of actually closed loops
  
  % To comptute the filter consitency (only when grount truth is
  % available).
  groundTruthAvailable=isfield(Parameters,'groundTruth');
  NEES=[]; 
  
  % continue from last step
  step=get(F,'nSteps');
  
  done=false;
  
  while ~done
   
    step=step+1;
    
    if Parameters.maxSteps>0
      done=(step>=Parameters.maxSteps);
    end
    
    if ~done
      % And now iterate reading the sensors and updating the filter
      % done is set to true when no more sensor readings are available
      [done R]=MoveRobot(R); % returns done=false whe no more sensor readings
    end

    if ~done

      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % Update the filter with the new sensor readings -> add a new state
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      [SR Sn]=GetOdometry(R);

      ck=cputime;
        F=Update(F,step,SR,Sn,redundantPose);
      timeSA(step)=(cputime-ck);

      if Parameters.plot
        if redundantPose
          pt=PlotFilter(F,Parameters);
        else
          pt=UpdateFilterPlot(F,step,pt,true);
        end
      end

      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % Use the filter information to detect possible points for loop
      % closuse (i.e., if the current pose is close enough)
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      switch Parameters.loopClosure
        case 'NoLoops'
          L=NoLoops(F);

        case 'AllLoops'
          L=AllLoops(F);

        case 'DetectLoops'
          ck=cputime;
            L=DetectLoops(F,Parameters.matchArea,Parameters.pdRange);
          timeS(step)=(cputime-ck);

        case 'TreeDetectLoops'
          if step>1
            pd=GetPoseData(F,step-1);
            if isa(pd,'PoseData')
              % if 'step' is not one of the removed steps
              ck=cputime;
                T=InsertInTree(T,pd);
              timeT(step)=(cputime-ck);
            end
          end
          ck=cputime;
            L=TreeDetectLoops(F,T,Parameters.matchArea,Parameters.pdRange);
          timeS(step)=(cputime-ck);
        otherwise
          error('Unknow loop closure procedure');
      end

      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % Check if at least one of the sensors confirm the loop closure
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      n=size(L,2);

      NPL=NPL+n; % number of tentative loop closures
      
      redundantPose=false;
      
      ncl=0; % number of closed loops in this time slice

      % Compute the information gain for all the tentative links 
      allI=cellfun(@(n)(InformationGain(F,Parameters.noiseLoopClosure,n.step)),L);
      
      while n>0
      
        % select the maximally informative link.
        [I mi]=max(allI);
        link=L{mi};
        cl=false; % the loop is not closed so far
        
        % link to the neighbouring pose
        ShowLinkToNeighbour(F,Parameters,pt,step,link.step,'y');

        if (I>Parameters.igRange(2)) && ~PreviousPose(F,link.step)

          %  link to the neighbouring informative pose
          ShowLinkToNeighbour(F,Parameters,pt,step,link.step,'b');

          % Look for the sensor that confirm the loop closure link
          j=1;
          nSLC=0; % Number of sensors that confirm the loop closure

          while (j<=numSensors) && (I>Parameters.igRange(2))
            [c d Sn]=SensorLoopCloses(get(R,'sensor',j),link.step,step);

            if c
              % If the sensor confirms the loop
              % we compute the true information gain
              Is=InformationGain(F,Sn,link.step);
              I=min(I,Is);
              
              if Is>Parameters.igRange(2)
                % This sensor confirms the loop and it is truly
                % informative: Store the link data (link mean and noise)
                % according to this sensor
                nSLC=nSLC+1;
                dSLC{nSLC}=d;
                sSLC{nSLC}=Sn;
              end
            end
            j=j+1;
          end

          % if at least one sensor confirms the infomrative link
          if nSLC>0
            cl=true; % the link is actually confirmed and the loop closed
            
            % if at least one sensor confirms the loop closure, execute it.
            ck=cputime;
              F=UpdateLoop(F,link.step,dSLC,sSLC);
            timeLC(step)=(cputime-ck);

            ncl=ncl+1;

            totalIG=totalIG+I;

            if ~Parameters.quiet
              fprintf('  Loop closure: [%u %u] ig:%f pd:%f -> ',link.step,step,I,link.pd);
              display(dSLC{1});
            end

            if Parameters.plot
              % Replot the filter after the loop closure
              pt=PlotFilter(F,Parameters);
            end
          end
        end % information is large enough

        % If the pose is close to another pose and the change in the map when
        % linking the two poses is small -> the current pose is redundant
        if (link.pd>Parameters.pdRange(2)) && (I<Parameters.igRange(1))
          redundantPose=true;
        end
        
        L(mi)=[]; % Removes element mi from L
        
        if cl
          % If the link resulted in a loop we have to recompute the
          % information gain
          allI=cellfun(@(n)(InformationGain(F,Parameters.noiseLoopClosure,n.step)),L);
        else
          allI(mi)=[]; % Removes element mi from allI
        end
        
        n=size(L,2); % n=n-1
         
      end % while n>0
        
      if ncl>0
        redundantPose=false;
        if strcmp(Parameters.loopClosure,'TreeDetectLoops')
          % If we closed a loop, we have to update the tree
          pd=GetPoseData(F);
          ck=cputime;
            T=UpdateTree(T,F,pd);
          timeT(step)=(cputime-ck);
        end
      end

      NCL=NCL+ncl; %number of loops actually closed

      if groundTruthAvailable
        NEES(step)=ComputeNEES(F,Parameters.groundTruth);
      end
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      pt=HideLinkToNeighbour(Parameters,pt);

    end % if ~done
  
  end % while
 
  Result.Filter=F; % Filter at the end of the simulation
  Result.Robot=R; % Robot used in the simulation
  Result.Tree=T; % Tree used for data association, possibly empty
  Result.Parameters=Parameters; % Parameters used in the simulation
  
  Result.timeS=timeS; % Seach time during data association
  Result.timeT=timeT; % Time used building and updating the tree
  Result.timeSA=timeSA; % Time used in state augmentation (one value per iteration)
  Result.timeLC=timeLC; % Time used in loop closure (one value per iteration)
  Result.timeF=sum(timeSA)+sum(timeLC); % Total time used in filter operations
  Result.time=Result.timeF+sum(Result.timeS)+sum(Result.timeT); % Total time used in the simulation
  Result.numClosedLoops=NCL; % Number of closed loops
  Result.numProposedLoops=NPL; % Number of proposed loops
  
  Result.totalIG=totalIG; % Sum of the information gain for all loops
  Result.NEES=NEES; % NEES per iteration
  
  if ~Parameters.quiet
    fprintf('\nSimulation done:\n');
    fprintf(' Execution time: %f s (%f s per step)\n',Result.time,Result.time/step);
    fprintf('   NN search time        : %f\n',sum(Result.timeS));
    if strcmp(Parameters.loopClosure,'TreeDetectLoops')
    fprintf('   Tree update time      : %f\n',sum(Result.timeT));
    end
    fprintf('   State Augmenation time: %f\n',sum(Result.timeSA));
    fprintf('   Loop Closure time     : %f\n',sum(Result.timeLC));
    if Result.numProposedLoops>0
      fprintf(' Closed vs. Proposed loops: %u/%u = %.2f %%\n',Result.numClosedLoops,Result.numProposedLoops,Result.numClosedLoops/Result.numProposedLoops*100);
    else
      fprintf(' No loop closure was proposed\n');
    end
  end
  