function N=TreeDetectLoops(F,T,matchArea,pdRange)
% Propose loops based on the distance between poses using a tree for the search. 
%
% One of the procedures to detect the possible loop closure
% points from the filter information.
%
% This implements the same algorithm as DetectLoops but the search for
% neighbouring poses is done using a tree and thus it is faster.
%
% Parameters
%   F: The filter from where to get the information.
%   T: The tree of poses to use in the search.
%   matchArea: Area around the current robot's pose where to search for
%              neighbouring poses. This is a pose displacement expressed
%              in the frame of the current robot's pose.
%   pdRange: Interval [p1,p2]. If a pose has probabiliby higher than p1
%            of being in the matchArea, it is consired a neighbouring
%            pose for the current robot's pose. 
%            If the probability is above p2, the it is considered a very
%            close neighbouring pose.
%            This is 's' in the paper but here we have a range for this
%            parameter and not a single value.

% Outputs:
%   N: A cell array with the neighbours where each element is a struct 
%      with two field
%       step: The identifier of the neighbour pose
%       pd: The neighbouring probability (always larger than pdRange(1).
% 
% See also NoLoops, AllLoops, DetectLoops.

  r=get(T,'root');
  if r==0
    N={};
  else
    S=SearchInTree(T,F,matchArea,pdRange(1),r);
 
    nn=size(S,2); % number neighbours
    N=cell(1,nn);
    for t=1:nn
      step=get(S{t},'id');
      N{t}.step=step;
      N{t}.pd=ProbDistance2PoseBelowThreshold(F,step,matchArea,@GetRelativeDistance2Pose);
    end
  end
  
