function out=get(F,field)
% Generic get function for filters.
%
% Retives information from a filter. This basically returns the
% informations common to all filters.
%
% Possible fields to query are:
%   - initialState: The Gaussian with the estimation of the initial pose
%        of the robot.
%   - nSteps: Simulation steps executed by the filter.
%   - nStates: The number of poses in the trajectory estimated by the
%        filter. In general we store one pose for each step but some 
%        filters (TRO filter) remove redundant poses. In this case 
%        we have less estates than steps.
%   - dim: Dimensionality of the poses in the estimated trajectory.
%   - covariances: Returns a cell array with the marginal covariances for
%        each one of the poses stored in the filter. Note that different
%        filters provide different covariance estimations.
%   - crossCovariances: Retuns a cell array with the cross-covariances
%        between all the poses stored in the filter and the current 
%        robot's pose.
%   - loops: Cell array with all loops closed so far.
%   - nLoops: The number of loops closed so far.

  switch field
    case  'initialState'
      out=F.initEstimation;
      
    case 'nStates'
      out=F.nStates;
      
    case 'nSteps'
      out=F.nSteps;
      
    case 'dim'
      out=F.dim;
      
    case 'covariances'
      out=cell(1,F.nStates);
      for i=1:n
        out{i}=get(GetPoseEstimation(F,F.State2Step(F,i)),'covariance');
      end
      
    case 'crossCovariances'
      out=zeros(F.nStates*F.dim,F.dim);
      for i=1:n
        ind=(i-1)*dim+1:i*dim;
        out(ind)=GetCrossCovarianceWithPose(F,F.State2Step(F,i));
      end
      
    case 'loops'
      out=F.loops;
      
    case 'nLoops'
      out=size(F.loops,2);
      
    otherwise
      error('Unknown field in Filter get');
  end