function FF=P_TRO(varargin)
% Constructor of TRO Filters for Pose SLAM.
%
% This is a sub-type of Pose SLAM filter (P_Filter), and thus it 
% inherits all methods from the P_Filter.
%
% This type of filter estimates the robot's trajectory representing it as
% Gaussian represented in the information form. 
% However, beside the information matrix and information vector, we need
% to store the mean (it is used as a linearization point for non linear
% transition and observation functions). Moreover, we have to store
% information to retrive the covariances for all poses and the
% cross-correlations between all poses and the last robot's pose.
% 
% The possible parameters are
%   - Another P_TRO. Copy constructor.
%   - A Gaussian given the initial estimation + the state recovery method 
%     that can be
%          2 -> Our method 
%          1 -> Get whole Sigma 
%          0 -> Get Sigma column by column
%
% See also P_Filter, P_AllSensors, P_OneSensor, P_EIF, P_FusionEKF, P_EKF.

  switch nargin
    case 1
      if isa(varargin{1},'P_TRO')
        FF=varargin{1};
      else
        % default state recovery method 2 (out method)
        FF=P_TRO(varargin{1},2);
      end
      
    case 2
      if isa(varargin{1},'Gaussian')
        % TRO filter can be seen as a sub-type of EIF but the
        % implementation as a different type of filter is more efficient
        F=P_Filter(varargin{1});
        
        dim=get(F,'dim');
        
        if get(varargin{1},'volume')<1e-9
          FF.Lambda=sparse(1e3*eye(dim)); % this should be infinite
        else
          FF.Lambda=sparse(get(varargin{1},'invCovariance'));
        end
        
        FF.mu=get(varargin{1},'mean'); % used as a linealization point
        FF.eta=FF.Lambda*FF.mu;
        
        % Information associated with each pose: mu, marginal sigma, and
        % the necessary information to recover the cross covariance with
        % respect to last pose (\Phi_i in the paper).
        FF.PoseData={PoseData(FF.mu,FF.Lambda\eye(dim),eye(dim),get(F,'nSteps'))};
        
        % F is the matrix used to correctly interpret the cross
        % correlation data
        FF.F=eye(dim);
        
        % State recovery method when closing a loop
        %  2 -> Our method
        %  1 -> Get whole Sigma
        %  0 -> Get Sigma column by column
        if isa(varargin{2},'double')
          FF.stateRecoveryMethod=varargin{2};
        else
          error('Wrong parameter type in P_TRO creation');
        end
        
        % To get statistics about cholmod2
        FF.cm2ne=[];
        FF.cm2t=[];
        FF.cm2mb=[];
        
        FF=class(FF,'P_TRO',F);
      else
        error('Wrong parameter type in P_TRO creation');
      end
      
    otherwise
      error('Wrong number of parameters in P_TRO creation');
  end
