function FF=TRO_Prediction(FF,d,Q,overwritePose)
% Prediction Step.
%
% This is used when extending the trajectory (i.e., the state), with a
% new pose.
% It updates the information matrix, the information vector, the mean 
% (necessary as a linearization point for all filter operations) and
% updates F, the accumulation of linearizations of the displacement
% function (see Relative2AbsoluteJacobian) that is latter used to compute
% cross covariances of poses w.r.t. the new robot's pose.
%  
% Parameters
%   FF: The filter to update
%   d: The sensor reading providing the position of the new robot's pose
%      from its predecesor pose.
%   Q: The noise of the sensor providing 'd'.
%   overwritePose: If true, the last pose in the filter is removed and
%                  replaced by the new one. In this way the number of
%                  estimated elements actually does not grow and we end
%                  up with less states than simulation steps.
% Outputs:
%   FF: The updated filter with the new pose.
%
%  See also EKF_Prediction, EKF_Update, Update, Relative2AbsoluteJacobian.

  ne=size(FF);
  dim=get(FF.P_Filter,'dim'); % dimensionality of the pose space
  nStates=get(FF.P_Filter,'nStates');
  step=get(FF.P_Filter,'nSteps');
  
  if overwritePose
    lastState=nStates-1; % Last valid state in the filter (in this case last
    % state minus one)
    newState=nStates;    % Index where to store the new state (in this case)
    % we re-use last state)
    
    last=(lastState-1)*dim+1:lastState*dim;
    new=(newState-1)*dim+1:newState*dim;
    
    % First we undo the effect of the last _Prediction
    iQw12=FF.Lambda(new,new);
    Qw12=iQw12\eye(dim);
    Jp12=-Qw12*FF.Lambda(new,last);
    
    FF.Lambda(last,last)=sparse2(FF.Lambda(last,last)-sparse2(Jp12'*iQw12*Jp12));
    
    etaN=FF.eta(new);
    FF.eta(last)=FF.eta(last)+Jp12'*etaN;
    
    FF.F=FF.F*(Jp12\eye(dim))';
    
    % Now we define the data necessary to execute the prediction
    % below
    % Qw: Noise in global coordinates
    % P1d:  last pose in the trajectory that we keep (as doubles).
    % Jp: Jacobian of Relative2Absolute w.r.t. the last pose in the
    %     trajectory that we keep.
    % newPose: New robot position (mean).
    P1d=FF.mu(last); % Last know pose that we will keep
    P1=Pose(P1d);
    
    P2=Pose(FF.mu(new));% Pose that will be removed
    
    P3=Relative2Absolute(P2,d); % Pose to connect directly with P1
    newPos=double(P3);
    
    [Jp23 Jd23]=Relative2AbsoluteJacobian(P2,d);
    
    Jp=Jp23*Jp12;
    
    Qw=Jp23*Qw12*Jp23'+Jd23*Q*Jd23';
  else
    lastState=nStates; % Last valid state in the filter
    newState=nStates+1; %  Index where to store the new state
    
    last=(lastState-1)*dim+1:lastState*dim;
    new=(newState-1)*dim+1:newState*dim;
    
    P1d=FF.mu(last);
    P1=Pose(P1d); % mean for the previous pose (used as a linealization
    % point
    
    % Jacobians used to linarly propagate the error from previous poses
    % to the new one
    [Jp Jd]=Relative2AbsoluteJacobian(P1,d);
    
    Qw=Jd*Q*Jd'; % displacement noise in global coordinates
    
    newPos=double(Relative2Absolute(P1,d));
    
    % make room for the new data
    Zk=zeros(dim,1);
    FF.eta=[FF.eta ; Zk];
    FF.mu=[FF.mu; Zk];
    
    Znk=sparse2(ne,dim);
    Zkk=sparse2(dim,dim);
    FF.Lambda=sparse2([FF.Lambda Znk ; Znk' Zkk ]);
  end
  
  v=newPos-Jp*P1d;
  
  iQw=Qw\eye(dim);
  etaN=iQw*v;
  
  FF.eta(last)=FF.eta(last)-Jp'*etaN;
  FF.eta(new)=etaN;
  
  FF.Lambda(last,last)=sparse2(FF.Lambda(last,last)+sparse2(Jp'*iQw*Jp));
  
  FF.Lambda(last,new)=sparse2(-Jp'*iQw);
  FF.Lambda(new,last)=sparse2(-iQw*Jp);
  FF.Lambda(new,new)=sparse2(iQw);
  
  FF.mu(new)=newPos;
  
  Sp=get(FF.PoseData{lastState},'covariance'); % Estimation for the previous pose
  FF.PoseData{lastState}=set(FF.PoseData{lastState},'crossCovarianceData',Sp*(FF.F\eye(dim)));
  
  FF.PoseData{newState}=PoseData(newPos,Jp*Sp*Jp'+Qw,eye(dim),step+1);
  
  FF.F=FF.F*Jp';
  
  
