function FF=TRO_Update(FF,i,d,R)
% Filter update step.
%
% This is used when establishing a loop between the current robot pose
% and one of the previous poses, i. This can be the previous pose or one of
% the poses before it. In the first case we are basically forming a local
% loop with limited effects (only affects the current robot pose) and
% in the second we have a loop that can affect a significant part of the
% trajectory.
% Besides the update of the mean (it is the linealization point used in
% all EIF operations) and cross covariances of all the poses w.r.t. the
% last robot's pose and the marginal covariances of all poses w.r.t the
% current one.
% All together can be expensive but this is compensated if we are
% cautious with the loop formation and we only form reliable and
% informative loops. The cost of closing a loop is then amortized over 
% long periods without loop formations.
%
% Parameters
%    FF: The filter to update.
%    i; The pose (state) to be linked with the current robot's pose to
%       forma a loop. 
%    d: Displacement from pose 'i' to the current robot's pose 'n'
%       according to the sensors readings.
%    R: Noise of the sensor
% Outputs
%    FF: The updated filter.
%
% See also EKF_Prediction, EKF_Update, UpdateLoop.

  %  stateRecoveryMethod
  % 0 -> Get Sigma column by column
  % 1 -> Get whole Sigma
  % 2 -> Our method
  
    ck=cputime;
    
    nStates=get(FF.P_Filter,'nStates');
    
    if i==nStates
      error('Self loop in  Update')
    end
    
    dim=get(FF.P_Filter,'dim'); % Dimensionality of the pose space
    rangeI=(dim*(i-1)+1):(dim*i);
    rangeN=(dim*(nStates-1)+1):(dim*nStates);
    
    ne=size(FF);
    
    mI=FF.mu(rangeI);
    Pi=Pose(mI);
    
    mN=FF.mu(rangeN);
    Pn=Pose(mN);
    
    [Ji Jn]=Absolute2RelativeJacobian(Pi,Pn);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if FF.stateRecoveryMethod==2
      % The state recovery proposed in the paper.
      
      % Compute the innovation and the inverse of its Cholesky factor.
      M=GetMarginalCovariance(FF,State2Step(FF,i));
      J=[Jn Ji];
      
      S=J*M*J'+R;
      S=(S+S')/2; % This enhances numerical stability
      L=chol(S,'lower'); % S=L*L'
      iL=L\eye(dim); % L is triangular -> this is easy.
      
      % Compute the i-th block column of the current covariance matrix
      z=sparse2(ne,dim);
      z(rangeI,:)=sparse2(1:dim,1:dim,1);
      
      [Si stats]=cholmod2(FF.Lambda,z); % the i-th column
      mb=stats(5);
      
      % Compute the n-th block column of the current covariance matrix
      Sn=zeros(ne,dim);
      ind=1:dim;
      for j=1:nStates-1
        Sn(ind,:)=get(FF.PoseData{j},'crossCovarianceData');
        ind=ind+dim;
      end
      Sn=Sn*FF.F;
      Sn(rangeN,:)=M(1:dim,1:dim);
      
      % Compute the block column used to update the covariance and the mean
      T=[Sn Si]*J'*iL';
      
      % Update the mean
      yt=double(d-Absolute2Relative(Pi,Pn));
      mu=FF.mu+T*(iL*yt);
      
      % Compute the last column of the new covariance matrix
      Tn=T(rangeN,:);
      nSn=Sn-T*Tn';
      
      % Compute the new diagonal of the covariance matrix
      ind=1:dim;
      for j=1:nStates-1
        Tj=T(ind,:);
        mu(ind)=double(Pose(mu(ind)));
        FF.PoseData{j}=set(FF.PoseData{j},'updateInfo',mu(ind),Tj*Tj',nSn(ind,:));
        ind=ind+dim;
      end
      mu(rangeN)=double(Pose(mu(rangeN)));
      FF.PoseData{nStates}=set(FF.PoseData{nStates},'updateInfo',mu(rangeN),Tn*Tn',eye(dim));
      
      clear Sn Si T nSn;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    iR=sparse2(R\eye(dim));
    
    H=sparse2(dim,ne);
    H(:,rangeI)=Ji;
    H(:,rangeN)=Jn;
    
    FF.Lambda=sparse2(FF.Lambda+H'*iR*H);
    if FF.stateRecoveryMethod~=2
      FF.eta=FF.eta+H'*iR*(double(d-Absolute2Relative(Pi,Pn))+H*FF.mu);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if FF.stateRecoveryMethod==1
      % A single cholmod2. This is efficient but only works for small
      % problems due to memory requirements. This is why we can not use
      % it as a default mode.
      
      [S stats]=cholmod2(FF.Lambda,sparse2(1:ne,1:ne,1));
      mb=stats(5);
      
      mu=S*FF.eta;
      
      ind=1:dim;
      for j=1:nStates
        mu(ind)=double(Pose(mu(ind)));
        FF.PoseData{j}=set(FF.PoseData{j},'info',mu(ind),S(ind,ind),S(ind,rangeN));
        ind=ind+dim;
      end
      
      clear S;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if FF.stateRecoveryMethod==0
      % One system per column. This is the standard approach since it can
      % be use despite the size of the problem.
      %
      % Observe that we solve one separate system for block colum of the
      % covariance matrix.
      % In principle we could decompose the information matrix once
      %           R=lchol(FF.Lambda);
      % and then recover each block column of the covariance as
      %           y=R\z
      %           Sj=R'\y
      % with the corresponding z.
      % However in our experiments this only results in a marginal gain in
      % time while factor R is not necessarily sparse resulting thus in a
      % large memory use (such as when solving for the whole sigma).
            
      % In a first system we recover the my and the last block column of
      % the covariance matrix (the one with the marginal covariance for the
      % current robot pose and the cross-correlations betwen this pose and
      % all the poses already visited).
      z=sparse2(ne,dim+1);
      z(rangeN,1:dim)=sparse2(1:dim,1:dim,1);
      z(:,dim+1)=FF.eta;
      
      [Sn stats]=cholmod2(FF.Lambda,z); % the last column + mean
      mb=stats(5);
      
      mu=Sn(:,dim+1);
      mu(rangeN)=double(Pose(mu(rangeN)));
      FF.PoseData{nStates}=set(FF.PoseData{nStates},'info',mu(rangeN),Sn(rangeN,1:dim),eye(dim));
      
      % We now solve for each block-column of the covariance matrix to keep
      % the diagonal only.
      ind=1:dim;
      for j=1:nStates-1
        
        z=sparse2(ne,dim);
        z(ind,:)=sparse2(1:dim,1:dim,1);
        
        [Sj stats]=cholmod2(FF.Lambda,z); % the j-th column
        mb=max(mb,stats(5));
           
        mu(ind)=double(Pose(mu(ind)));
        FF.PoseData{j}=set(FF.PoseData{j},'info',mu(ind),Sj(ind,:),Sn(ind,1:dim));
        
        clear Sj;
        
        ind=ind+dim;
      end
      
      clear Sn;
      
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    FF.mu=mu;
    FF.eta=FF.Lambda*FF.mu;
    FF.F=eye(dim);
    
    FF.cm2ne=[FF.cm2ne ne];
    FF.cm2t= [FF.cm2t  cputime-ck];
    FF.cm2mb=[FF.cm2mb mb];

