function [sLow sUp]=ProbDistance2PoseDataAboveThreshold(PD,F,t,pt)
% p=ProbDistance2PoseDataAboveThreshold(PD,F,t,pt)
%
% Computes the probability of the relative displacement between the 
% current robot's pose and a given PoseData, PD, to be lower than 't'.
%
% Note that the threshold 't' has one entry for each dimension of the
% pose space (this is v_r in the paper).
%
% This function perform a short-circuit evaluation of the conditions 
% for all the dimensions of the relative displacement. 
%
% The probability is estimated as a range [sLow sUp].
% The evaluation is performed while sUp is above the given
% threshold 'pt': as soon sUp is below 'pt' for any dimension, we know for
% sure that the tested pose is too far (at least in that dimension) from
% the current pose and it does not need to be used for sensor registration.
% If after all the evaluations sLow is above 'st' all the poses in the
% PoseData object are neighbours of the current pose (nor further
% processing is needed). If sUp is below 'pt' none of the poses represented
% in PoseData are neighbours. If 'pt' is inside the range [sLow,sUp], the
% elements in PoseData need to be further refined.
%
% For the special case of orientations, intervals in PoseData larger than 
% 2*pi are directly produce [sLow sUp]=[0,1] meaning that they have to be
% further refined.
%
% Note that for PoseData objets including a single pose sLow=sUp.
%
% This function implements equations in section VI-A in the paper. It
% is hard to follow because interval expressions are 
% re-ordered to make them more efficient and to avoid overestimation 
% (a problem typical of interval arithmtics).
%
% See also ProbDistance2PoseBelowThreshold.

  if ~PD.isInterval
    sUp=ProbDistance2PoseBelowThreshold(F,PD.id,t,@GetRelativeDistance2Pose);
    sLow=sUp;
  else
    dim=get(F,'dim');
 
    nSteps=get(F,'nSteps');
    PoseN=GetPoseMean(F,nSteps);
    
    o1=PoseN(3);
    c1=cos(o1);
    s1=sin(o1);
    
    R=[[ c1  s1 0];
       [-s1  c1 0];
       [  0   0 1]];
     
    dx=(PD.m(1)-PoseN(1));
    dy=(PD.m(2)-PoseN(2));

    dx2=dx.^2;
    dy2=dy.^2;
    dxdy=dx.*dy;

    S11=GetPoseCovariance(F,nSteps);

    FF=get(F,'F');
    Fa=FF(3,1);
    Fb=FF(3,2);

    T1=zeros(dim);
    T1(1,3)=s1;
    T1(2,3)=c1;

    T2=zeros(dim);
    T2(1,3)=-c1;
    T2(2,3)= s1;

    A1=diag(T1*S11*T1');
    A2=diag(T2*S11*T2');
    A3=2*diag(T1*S11*T2');
    A4=2*diag(R*S11*T1');
    A5=2*diag(R*S11*T2');
    A6=diag(R*S11*R');

    s1c1=s1*c1;
    s12=s1^2;
    c12=c1^2;
    
    positive=Interval(1e-100,inf);
    
    PD_CS_1_3=PD.CS(1,3);
    PD_CS_2_3=PD.CS(2,3);
    ct1=Scale(PD_CS_1_3,s1c1);
    B1_1=-ct1+Scale(PD_CS_2_3,-s12);
    
    ct2=Scale(PD_CS_2_3,s1c1);
    B2_1= ct2+Scale(PD_CS_1_3,c12);
    
    PD_CS_1_2=PD.CS(1,2);
    PD_CS_2_1=PD.CS(2,1);
    PD_CS_1_1=PD.CS(1,1);
    PD_CS_2_2=PD.CS(2,2);
    ct3=Scale(PD_CS_1_2+PD_CS_2_1,s1c1);
    B3_1=Scale(PD_CS_1_1,c12)+Scale(PD_CS_1_3, Fb*s1c1+Fa*c12)+ct3+Scale(PD_CS_2_2,s12)+Scale(PD_CS_2_3,Fb*s12+Fa*s1c1);

    PD_S_1_1=PD.S(1,1);
    PD_S_2_2=PD.S(2,2);
    ct4=Scale(PD.S(1,2),2*s1c1);
    C1_1=Scale(PD_S_1_1,c12)+ct4+Scale(PD_S_2_2,s12);
    
    ct5=Offset(Scale(B3_1,-2)+C1_1,A6(1));
    ct6=Offset(Scale(B1_1,2),A4(1));
    ct7=Offset(Scale(B2_1,2),A5(1));
    S1=Scale(dx2,A1(1))+Scale(dy2,A2(1))+Scale(dxdy,A3(1))+ct6.*dx+ct7.*dy+ct5;
    Sx=Intersection(S1,positive);

    m_1=Scale(dx, c1)+Scale(dy,s1);

    rtU=( t(1)-m_1)/sqrt(2*Sx);
    rtL=(-t(1)-m_1)/sqrt(2*Sx);
    sUp =min(0.5*(erf(Upper(rtU))-erf(Lower(rtL))),1);
    sLow=max(0.5*(erf(Lower(rtU))-erf(Upper(rtL))),0);
    
    if (sUp>pt)
      B1_2= ct1+Scale(PD_CS_2_3,-c12);

      B2_2=-ct2+Scale(PD_CS_1_3,s12);

      B3_2=Scale(PD_CS_1_1,s12)+Scale(PD_CS_1_3,-(Fb*s1c1-Fa*s12))-ct3+Scale(PD_CS_2_2,c12)+Scale(PD_CS_2_3,Fb*c12-Fa*s1c1);

      C1_2=Scale(PD_S_1_1,s12)-ct4+Scale(PD_S_2_2,c12);

      ct8=Offset(Scale(B3_2,-2)+C1_2,A6(2));
      ct9=Offset(Scale(B1_2,2),A4(2));
      ct10=Offset(Scale(B2_2,2),A5(2));
      S2=Scale(dx2,A1(2))+Scale(dy2,A2(2))+Scale(dxdy,A3(2))+ct9.*dx+ct10.*dy+ct8;
      Sy=Intersection(S2,positive);
 
      m_2=Scale(dx,-s1)+Scale(dy,c1);

      rtU=( t(2)-m_2)/sqrt(2*Sy);
      rtL=(-t(2)-m_2)/sqrt(2*Sy);
      sUp =min(min(0.5*(erf(Upper(rtU))-erf(Lower(rtL))),1),sUp);
      sLow=min(max(0.5*(erf(Lower(rtU))-erf(Upper(rtL))),0),sLow);
      
      if (sUp>pt)
        if Diameter(PD.m(3))<=2*pi
          S3=Offset(Scale(dx2,A1(3))+Scale(dy2,A2(3))+Scale(dxdy,A3(3))+Scale(dx,A4(3))+Scale(dy,A5(3))+Scale(PD.CS(3,3),-2)+PD.S(3,3),A6(3));
          So=Intersection(S3,positive);
  
          do=pi2pi(PD.m(3)-PoseN(3));
          
          rtU=( t(3)-do)/sqrt(2*So);
          rtL=(-t(3)-do)/sqrt(2*So);
          sUp =min(min(0.5*(erf(Upper(rtU))-erf(Lower(rtL))),1),sUp);
          sLow=min(max(0.5*(erf(Lower(rtU))-erf(Upper(rtL))),0),sLow);
        else
          sUp=1;
          sLow=0;
        end
      end
    end
  end
 